/*
 Ejemplo de comunicación ESP-NOW por Dani No www.esploradores.com

***ESTE SKETCH CORRESPONDE AL PAR ESCLAVO***

El sketch permite la recepción de una comunicación vía ESP-NOW de dos datos enviados
por dos módulos maestros y su envío a la plataforma THINGER.IO.

 ¡¡¡IMPORTANTE!!!

Para hacer funcionar la comunicación ESP-NOW si se tiene instalado en Arduino como gestor 
de tarjetas el: esp8266 by ESP8266 Community versión 2.3.0 (Se puede comprobar buscándolo en:
Herramientas->Placa:"NodeMCU 1.0(ESP-12E Module)"->Gestor de tarjetas..., es necesario editar 
el fichero: ~/Library/Arduino15/packages/esp8266/hardware/esp8266/2.1.0/platform.txt,
buscar "compiler.c.elf.libs", y añadir al final de la línea "-lespnow". 
*/

#include <ESP8266WiFi.h>       //Librería de conexión WiFi del módulo ESP8266
#include <ThingerESP8266.h>    //Librería de la plataforma thinger.io
extern "C" {
  #include <espnow.h>          //Comunicación ESP-NOW
}

// Parámetros del conexión con thinger.io
#define usuario "???????????"
#define device_Id "???????????"
#define device_credentials "???????????"
ThingerESP8266 thing(usuario, device_Id, device_credentials);

// Parámetros de conexión WiFi
const char WiFi_ssid[]="???????????";      //Nombre de red
const char WiFi_password[]="???????????";  //Clave de red

//***ESTRUCTURA DE LOS DATOS TRANSMITIDOS MAESTRO/ESCLAVO***//
//Se de establecer IGUAL en los pares maestros
struct ESTRUCTURA_DATOS {
  int dispositivo;
  float temperatura;
  float humedad;
};


//***VARIABLES GLOBALES***//
ESTRUCTURA_DATOS datos;
float DS18B20_Temperatura; 
float DHT22_Temperatura;
float DHT22_Humedad;


void setup() {

  // Inicialización de la WiFi para comunicarse con la API
  thing.add_wifi(WiFi_ssid, WiFi_password);

  //***INICIALIZACIÓN DEL PROTOCOLO ESP-NOW***//   
  if (esp_now_init()!=0) {
    ESP.restart();
  }
  delay(1);

  //***DECLARACIÓN DEL PAPEL DEL DISPOSITIVO ESP EN LA COMUNICACIÓN***//
  esp_now_set_self_role(2);   // 2=ESCLAVO

  // Inicialización de la lectura de datos desde la API
  thing["SENSORES"] >> [](pson& out){
    out["DS18B20-Temperatura"] = DS18B20_Temperatura; 
    out["DHT22-Temperatura"] = DHT22_Temperatura;
    out["DHT22-Humedad"] = DHT22_Humedad;
  };
}

void loop() {

  //***RECEPCIÓN DE LA COMUNICACIÓN ESP-NOW***//
  esp_now_register_recv_cb([](uint8_t *mac, uint8_t *array_datos, uint8_t len) {

    //***RECONVERSIÓN DE LA ESTRUCTURA DE DATOS***// 
    memcpy(&datos, array_datos, sizeof(datos));

    //***ASIGNACIÓN DE LOS DATOS A LAS VARIABLES DE THINGER.IO***// 
    if (datos.dispositivo ==1){
      DS18B20_Temperatura = datos.temperatura;
    }
    else if (datos.dispositivo ==2){
      DHT22_Temperatura = datos.temperatura;
      DHT22_Humedad = datos.humedad;
    }
    else {
      return;
    }  
  });
 
 thing.handle();
}
