/*
 Ejemplo de comunicación ESP-NOW por Dani No www.esploradores.com

***ESTE SKETCH CORRESPONDE AL PAR MAESTRO Nº2***
El sketch permite el envío de dos datos leidos por un sesor de Tª y humedad DHT22 al par esclavo conectado con THINGER.IO

 ¡¡¡IMPORTANTE!!!

Para hacer funcionar la comunicación ESP-NOW si se tiene instalado en Arduino como gestor 
de tarjetas el: esp8266 by ESP8266 Community versión 2.3.0 (Se puede comprobar buscándolo en:
Herramientas->Placa:"NodeMCU 1.0(ESP-12E Module)"->Gestor de tarjetas..., es necesario editar 
el fichero: ~/Library/Arduino15/packages/esp8266/hardware/esp8266/2.1.0/platform.txt,
buscar "compiler.c.elf.libs", y añadir al final de la línea "-lespnow". 
*/

#include <ESP8266WiFi.h>       //Librería de conexión WiFi del módulo ESP8266
#include "DHT.h"               //Librería de los sensores DHT11, DHT22, etc. 
extern "C" {
  #include <espnow.h>
}

// Parámetros del DHT 
#define DHTPIN D4       //Pin de conexión - GPIO02
#define DHTTYPE DHT22   //Modelo
DHT dht(DHTPIN, DHTTYPE);

//***ESTRUCTURA DE LOS DATOS TRANSMISIÓN MAESTRO/ESCLAVO***//
//Se de establecer IGUAL en el par esclavo
    struct ESTRUCTURA_DATOS {
    int dispositivo;
    float temperatura;
    float humedad;
};

//***VARIABLE GLOBAL***//
bool apagar=0;                  //Variable para condicionar la entrada en modo deep-sleep

void setup() {

  //***INICIALIZACIÓN DEL DHT22***//  
  dht.begin();

  //***INICIALIZACIÓN DEL PROTOCOLO ESP-NOW***//  
  if (esp_now_init()!=0) {
    ESP.restart();
  }
  delay(1);

  //***DECLARACIÓN DEL PAPEL DEL DISPOSITIVO ESP EN LA COMUNICACIÓN ESPNOW***//
  esp_now_set_self_role(1);   //1=MAESTRO

  //***EMPAREJAMIENTO CON EL ESCLAVO***//
  uint8_t mac_addr[6] = {0x5C, 0xCF, 0x7F, 0x03, 0x8C, 0xC4};    //STA MAC del ESCLAVO 
  esp_now_add_peer(mac_addr,2,7,NULL,0);
}

void loop() {

  //***DATOS A ENVIAR***//

  float t_NOverificada = dht.readTemperature();
  float h_NOverificada = dht.readHumidity();
 
  if (isnan(t_NOverificada) || isnan(h_NOverificada)) { //Verifica que las lecturas de Tª y humedad son correctas
    return;
  }
  else{
    ESTRUCTURA_DATOS datos;                     
    datos.dispositivo = 2;
    datos.temperatura = t_NOverificada;                 //Asigna valores correctos a las variables a enviar Tª y humedad
    datos.humedad = h_NOverificada;
        
    //***TRANSFORMACIÓN DE LA ESTRUCTURA DE DATOS EN UN ARRAY CON DATOS HOMOGENEOS TIPO uint8_t PARA PODER ENVIARLOS VIA ESP-NOW***//
    uint8_t array_datos[sizeof(datos)]; 
    memcpy(array_datos, &datos, sizeof(datos));
  
    //***ENVÍO DE LOS DATOS***//
    esp_now_send(NULL, array_datos, sizeof(datos));
    delay(1); 

    //***VERIFICACIÓN DE LA RECEPCIÓN CORRECTA DE LOS DATOS POR EL ESCLAVO***//
    esp_now_register_send_cb([](uint8_t* mac, uint8_t status) {
      if (status == 0){
        apagar=1;
      }
      else {
        return;
      }
    });

    if (apagar==1){
      ESP.deepSleep(60e6, WAKE_RF_DEFAULT);
    }
  }
}
